import 'package:comunic/forez/helpers/forez_group_helper.dart';
import 'package:comunic/helpers/groups_helper.dart';
import 'package:comunic/helpers/users_helper.dart';
import 'package:comunic/lists/group_members_list.dart';
import 'package:comunic/lists/users_list.dart';
import 'package:comunic/models/group_membership.dart';
import 'package:comunic/models/user.dart';
import 'package:comunic/ui/widgets/account_image_widget.dart';
import 'package:comunic/ui/widgets/async_screen_widget.dart';
import 'package:comunic/utils/account_utils.dart';
import 'package:comunic/utils/conversations_utils.dart';
import 'package:comunic/utils/intl_utils.dart';
import 'package:flutter/material.dart';

/// Forez directory screen
///
/// @author Pierre Hubert

enum _PopupMenuActions { PRIVATE_CONVERSATION }

class ForezDirectoryScreen extends StatefulWidget {
  @override
  _ForezDirectoryScreenState createState() => _ForezDirectoryScreenState();
}

class _ForezDirectoryScreenState extends State<ForezDirectoryScreen> {
  final _key = GlobalKey<AsyncScreenWidgetState>();

  UsersList _users;
  GroupMembersList _members;

  Future<void> _load() async {
    _members = await GroupsHelper.getMembersList(forezGroup.id);
    _users = await UsersHelper().getListWithThrow(_members.usersID);
  }

  @override
  Widget build(BuildContext context) => AsyncScreenWidget(
        onReload: _load,
        onBuild: onBuild,
        errorMessage: tr("Failed to load members list!"),
        key: _key,
      );

  Widget onBuild() => ListView.builder(
        itemBuilder: (c, i) => _ForezMemberTile(
          member: _members[i],
          user: _users.getUser(_members[i].userID),
          selectedAction: _doPopupMenuAction,
        ),
        itemCount: _members.length,
      );

  void _doPopupMenuAction(_PopupMenuActions action, User user) async {
    switch (action) {
      case _PopupMenuActions.PRIVATE_CONVERSATION:
        openPrivateConversation(context, user.id);
        break;
    }
  }
}

class _ForezMemberTile extends StatelessWidget {
  final User user;
  final GroupMembership member;
  final Function(_PopupMenuActions, User) selectedAction;

  const _ForezMemberTile({
    Key key,
    @required this.user,
    @required this.member,
    @required this.selectedAction,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) => ListTile(
        leading: AccountImageWidget(user: user),
        title: Text(user.fullName),
        subtitle: Text(member.membershipText),
        trailing: user.id == userID()
            ? null
            : PopupMenuButton(
                itemBuilder: (c) => popupMenuItems,
                onSelected: (a) => selectedAction(a, user),
              ),
      );

  List<PopupMenuItem<_PopupMenuActions>> get popupMenuItems => [
        PopupMenuItem(
          child: Text(tr("Private conversation")),
          value: _PopupMenuActions.PRIVATE_CONVERSATION,
        ),
      ];
}
